<?php

declare(strict_types=1);

namespace mirzaev\marina\controllers;

// Файлы проекта
use mirzaev\marina\controllers\core,
	mirzaev\marina\controllers\traits\converters;

// Discord framework
use Discord\Discord as discord,
	Discord\Parts\Channel\Message as message,
	Discord\Parts\Guild\Role as role,
	Discord\Parts\Guild\CommandPermissions as permissions,
	Discord\Parts\User\Member as member,
	Discord\Parts\Interactions\Interaction as interaction,
	Discord\WebSockets\Event as event,
	Discord\Parts\Interactions\Command\Command as command,
	Discord\Builders\MessageBuilder as _message,
	Discord\Builders\CommandBuilder as _command,
	Discord\Parts\Interactions\Command\Option as option;

// Встроенные библиотеки
use exception;

/**
 * Контроллер головного управления
 *
 * @package mirzaev\marina\controllers
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class index extends core
{
	use converters {
		utf8_to_extended_ascii as protected ascii;
	}

	/**
	 * Главная страница
	 *
	 * @param array $parameters Параметры запроса
	 *
	 * @return void
	 */
	public function index(array $parameters = []): void
	{
		$this->discord->on('init', function (discord $discord) {
			// Запись в буфер вывода (терминал)
			echo "Marina is ready!", PHP_EOL;

			// Сообщение: создано
			$discord->on(event::MESSAGE_CREATE, function (message $message, discord $discord) {
				// Игнорирование чат-роботов
				if ($message->author->bot) return;

				// Запись в буфер вывода (терминал)
				echo "{$message->author->username}: {$message->content}", PHP_EOL;

				if ($message->author->id === '490821645028687873' || mb_stristr($message->content, 'мимк') !== false) {
					// Сообщение от Дмитрия Демина (мимк) (vladimirbaryshkin) или содержит его псевдоним

					// Добавление реакции на его сообщение
					$message->react('😬');
				}

				if ($message->author->id === '380876873468608522' && rand(0, 100) < 6) {
					// Сообщение от Даниила Богданова (godandem) и шанс 5%

					// Отправка изображения: "надругался над даниилом богдановым"
					$message->channel->sendMessage(_message::new()->addFile('images/daniil.png'));
				}

				foreach (explode(' ', $message->content) as $word) {
					// Перебор слов из текста сообщения

					// Инициализация буфера символов для конвертации: UTF-8 -> ASCII+
					$buffer = [];

					// Конвертация проверяемых слов
					$marina = self::ascii('марина', $buffer);
					$marishka = self::ascii('маришка', $buffer);
					$marinochka = self::ascii('мариночка', $buffer);
					$marinushka = self::ascii('маринушка', $buffer);
					$marya = self::ascii('маря', $buffer);

					// Конвертация слова
					$_word = self::ascii(mb_strtolower($word), $buffer);

					if (
						levenshtein($_word, $marina, 2, 2, 1) < 3
						|| levenshtein($_word, $marishka, 2, 1, 2) < 4
						|| levenshtein($_word, $marinochka, 2, 1, 2) < 4
						|| levenshtein($_word, $marinushka, 2, 1, 2) < 4
						|| (levenshtein($_word, $marya, 3, 3, 1) < 3 && $word !== 'мария')
					) {
						$message->reply(_message::new()->setContent('ЗДАРОВА'));
					}
				}
			});

			// Пользователь: обновление
			$discord->on(event::GUILD_MEMBER_UPDATE, function (member $new, discord $discord, ?member $old = null) {
				// Запись в буфер вывода (терминал)

				// Инициализация реестра ролей пользователя
				$_roles = array_reverse($new->roles->toArray());

				/**
				 *
				 *  НАЧАЛО БЛОКА: "если ни одна роль не назначена, то назначить другие роли"
				 *
				 */

				// Инициализация реестров ролей
				$roles = require '../settings/discord/roles/xor.php';

				// Инициализация статуса о том, что ни одна из ролей в реестре необходимых ролей не назначена
				$fail = true;

				// Проверка на назначение хотя бы одной роли из реестра необходимых ролей
				foreach ($_roles as $role)
					if (in_array($role->id, $roles['required'], true)) $fail = false;

				// Если ни одна роль из реестра не назначена, то выдать роли из реестра ролей назначения, иначе удалить роли из реестра ролей назначения
				if ($fail) foreach ($roles['else'] as $role => $reason) $new->addRole($role, $reason);
				else foreach ($roles['else'] as $role => $reason) $new->removeRole($role, 'CONDITION NOT COMPLIED: ' . $reason);

				// Деинициализация неактуальных переменных
				unset($roles, $fail);

				/**
				 *
				 * КОНЕЦ БЛОКА: "если ни одна роль не назначена, то назначить другие роли"
				 *
				 */

				/**
				 *
				 *  НАЧАЛО БЛОКА: "если есть дубликаты ролей, то назначить доминирующую, если нет других доминирующих"
				 *
				 */

				// Инициализация реестра ролей
				$roles = require '../settings/discord/roles/xor_dublicates.php';

				// Инициализация реестра найденных основных специальностей
				$mains = [];

				// Инициализация буфера идентификатора основной роли (после обработки)
				$id;

				// Поиск основной специальности
				foreach ($_roles as $role)
					if (in_array($role->id, $roles, true)) $mains[] = $role;

				// Подсчёт количества найденных основных специальностей
				$amount = count($mains);

				if ($amount > 1) {
					// Найдено несколько основных специальностей

					// Если не передано старое состояние аккаунта, то пропустить проверку старого состояния
					if (is_null($old)) goto deduplication;

					// Поиск добавленных ролей
					$diff = array_values(array_diff($_roles, $old->roles->toArray()));

					if (count($diff) === 1 && in_array($diff[0]->id, $roles, true)) {
						// Добавлена одна роль и она является доминирующей

						// Удаление всех основных ролей кроме добавленной
						foreach ($mains as $role)
							if ($role->id === $diff[0]->id) continue;
							else $new->removeRole($role);
					} else {
						// Добавлено несколько ролей, либо добавлена одна роль, но она не является доминирующей

						deduplication:

						for ($i = $amount - 1; $i > 0; --$i) {
							// Перебор реестра найденных основных ролей

							// Удаление основной роли (будут удалены все кроме первой в списке, то есть самой приоритетной в discord)
							$new->removeRole($mains[$i]);

							// Инициализация основной роли
							$id = $mains[0]->id;
						}
					}
				} else if ($amount === 1) $id = $mains[0]->id;
				else if ($amount === 0) {
					// Не найдена основная специальность

					foreach ($_roles as $role) {
						// Перебор ролей

						if (array_key_exists($role->id, $roles)) {
							// Найдена вторичная специальность

							// Замена вторичной специальности основной специальностью
							$new->removeRole($role->id, 'Main specialty chosen');
							$new->addRole($roles[$role->id], 'Main specialty chosen');

							// Реинициализация основной роли
							$id = $roles[$role->id];

							// Завершение выполнения (успех)
							break;
						}
					}
				}

				if (isset($id)) {
					// Найден идентификатор основной роли

					// Инициализация дубликата основной роли (вторичная роль)
					$secondary = array_flip($roles)[$id];

					foreach ($_roles as $role) {
						// Перебор ролей

						if ($role->id === (string) $secondary) {
							// Найдена вторичная специальность дублирующая основную

							// Замена вторичной специальности основной специальностью
							$new->removeRole($secondary, 'Main specialty chosen');

							// Завершение выполнения (успех)
							break;
						}
					}
				}

				// Деиницализация неактуальных перменных
				unset($roles, $mains, $id, $amount);

				/**
				 *
				 * КОНЕЦ БЛОКА: "если ни одна роль не назначена, то назначить другие роли"
				 *
				 */
			});

			$discord->application->commands->save(new command($discord, [
				'name' => 'dreamers',
				'description' => 'BEBRA DREAMERS',
				'options' => [(new option($discord))
					->setName('invite')
					->setDescription('Send an invitation link to "BEBRA DREAMERS"')
					->setType(option::SUB_COMMAND)]
			]));

			$discord->listenCommand(['dreamers', 'invite'], function (interaction $interaction) {
				// Игнорирование чат-роботов
				if ($interaction->user->bot) return;

				// Отправка ссылки с приглашением присоединиться на сервер BEBRA DREAMERS
				if ($interaction->data->options->has('invite')) $interaction->respondWithMessage(_message::new()->setContent('https://discord.bebra.team'));
			});
		});

		$this->discord->run();
	}
}
